'use strict';

require('babel-polyfill');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _genBase = require('../genBase');

var _genBase2 = _interopRequireDefault(_genBase);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _pify = require('pify');

var _pify2 = _interopRequireDefault(_pify);

var _package = require('../../package.json');

var _package2 = _interopRequireDefault(_package);

var _utils = require('../utils');

var _utils2 = _interopRequireDefault(_utils);

var _yosay = require('yosay');

var _yosay2 = _interopRequireDefault(_yosay);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

module.exports = _genBase2.default.extend({
  initializing: function initializing() {
    this.host = this.options.host || 'localhost';
    this.port = this.options.port || 3000;
    this.appDir = this.options['app-dir'] || 'app';
    this.unitTestDir = this.options['unit-test-dir'] || 'app';
    this.skipController = this.options['skip-controller'] || false;

    console.log(_chalk2.default.yellow('Using host: ' + this.host));
    console.log(_chalk2.default.yellow('Using port: ' + this.port));
    console.log(_chalk2.default.yellow('Using app directory: ' + this.appDir));
    console.log(_chalk2.default.yellow('Using unit test directory: ' + this.unitTestDir));
    console.log(_chalk2.default.yellow('Route generator will ' + (this.skipController ? _chalk2.default.red('NOT ') : '') + 'create controllers'));
  },
  prompting: function prompting() {
    var _this = this;

    var done = this.async();
    this.log((0, _yosay2.default)('Welcome to ngPoly!'));

    // ask for app name
    // get preferred langugaes
    this.prompt([{
      name: 'appName',
      message: 'What is the app\'s name?',
      validate: function validate(input) {
        return !!input;
      }
    }, {
      type: 'list',
      name: 'ngversion',
      message: 'Which version of Angular should be used?',
      default: '1.4.*',
      choices: _utils2.default.getAngularVersions()
    }, {
      type: 'list',
      name: 'structure',
      message: 'Which structure should be used?',
      default: 'module-only',
      choices: _utils2.default.getModuleStructures()
    }, {
      type: 'list',
      name: 'markup',
      message: 'Which is the preferred markup language?',
      default: 'jade',
      choices: _utils2.default.getMarkupLanguages()
    }, {
      type: 'list',
      name: 'appScript',
      message: 'Which is the preferred application scripting language?',
      default: 'js',
      choices: _utils2.default.getScriptLanguages()
    }, {
      type: 'confirm',
      name: 'controllerAs',
      message: 'Want to use Controller As syntax?',
      default: true
    }, {
      type: 'confirm',
      name: 'directiveTemplateUrl',
      message: 'Should directives be generated using a templateUrl (and markup file) instead of an inline template?',
      default: true
    }, {
      type: 'list',
      name: 'testScript',
      message: 'Which is the preferred test scripting language?',
      default: function _default(answers) {
        return answers.appScript;
      },
      choices: _utils2.default.getScriptLanguages()
    }, {
      type: 'list',
      name: 'testFramework',
      message: 'Which is the preferred unit testing framework?',
      default: 'jasmine',
      choices: _utils2.default.getTestFrameworks()
    }, {
      type: 'list',
      name: 'e2eTestFramework',
      message: 'Which is the preferred e2e testing framework?',
      default: function _default(answers) {
        return answers.testFramework;
      },
      choices: _utils2.default.getTestFrameworks()
    }, {
      type: 'list',
      name: 'style',
      message: 'Which is the preferred style language?',
      default: 'less',
      choices: _utils2.default.getStyleLanguages()
    }, {
      type: 'confirm',
      name: 'polymer',
      message: 'Should Polymer support be enabled?',
      default: false
    }, {
      type: 'list',
      name: 'framework',
      message: 'Should a framework be setup?',
      choices: function choices(answers) {
        return _utils2.default.getUIFrameworks(answers.ngversion);
      }
    }, {
      type: 'confirm',
      name: 'ngRoute',
      message: 'Should ngRoute be used instead of UI Router?',
      default: false
    }, {
      type: 'checkbox',
      name: 'bower',
      message: 'Which additional Bower components should be installed?',
      choices: function choices(answers) {
        return _utils2.default.getBowerComponents(answers.ngversion);
      }
    }], function (props) {
      // needs to be a string
      props.bower = props.bower.join(',');

      // attach answers to `this`
      _lodash2.default.merge(_this, props);

      done();
    });
  },
  configuring: function configuring() {
    var _this2 = this;

    // create a directory named `appName`
    this.destinationRoot(this.appName);

    // save config
    ['appScript', 'controllerAs', 'directiveTemplateUrl', 'e2eTestFramework', 'markup', 'ngRoute', 'structure', 'style', 'testFramework', 'testScript'].forEach(function (option) {
      return _this2.config.set(option, _this2[option]);
    });

    this.context = {
      pkg: _package2.default,
      appName: this.appName,
      structure: this.structure,
      ngversion: this.ngversion,
      appDir: this.appDir,
      appScript: this.appScript,
      testScript: this.testScript,
      markup: this.markup,
      style: this.style,
      unitTestDir: this.unitTestDir,
      host: this.host,
      port: this.port,
      moduleName: _utils2.default.lowerCamel(this.appName),
      polymer: this.polymer,
      framework: this.framework,
      testFramework: this.testFramework,
      e2eTestFramework: this.e2eTestFramework,
      ngRoute: this.ngRoute,
      bower: this.bower
    };

    // copy over common project files
    ['.bowerrc', '.editorconfig', '.eslintrc', '.jscsrc', '.jshintrc', '_bower.json', '_build.config.js', '_karma.config.js', '_package.json', '_protractor.config.js', 'gulp/analyze.js', 'gulp/_build.js', 'gulp/_test.js', 'gulp/watch.js'].forEach(function (file) {
      return _this2.copySimpleFile(file);
    });

    // files that need to be renamed when copied
    this.copySimpleFile('_gulpfile.js', 'Gulpfile.js');
    this.copySimpleFile('gitignore', '.gitignore');
    this.copySimpleFile('_readme.md', 'README.md');

    if (this.appScript === 'ts') {
      this.copySimpleFile('_tsd.json');
    }
  },
  writing: function writing() {
    var done;
    return regeneratorRuntime.async(function writing$(_context) {
      while (1) switch (_context.prev = _context.next) {
        case 0:
          done = this.async();

          // create main module and index.html

          this.copySimpleFile('_app.' + this.appScript, _path2.default.join(this.appDir, 'app-module.' + this.appScript));
          this.copySimpleFile('_app-routes.' + this.appScript, _path2.default.join(this.appDir, 'app-routes.' + this.appScript));
          this.copySimpleFile('_index.' + this.markup, _path2.default.join(this.appDir, 'index.' + this.markup));

          _context.next = 6;
          return regeneratorRuntime.awrap((0, _pify2.default)(_mkdirp2.default)(_path2.default.join(this.appDir, 'fonts')));

        case 6:
          _context.next = 8;
          return regeneratorRuntime.awrap((0, _pify2.default)(_mkdirp2.default)(_path2.default.join(this.appDir, 'images')));

        case 8:

          done();

        case 9:
        case 'end':
          return _context.stop();
      }
    }, null, this);
  },
  install: function install() {
    if (!this.options['skip-install']) {
      this.installDependencies();
      if (this.appScript === 'ts') {
        this.log('Running ' + _chalk2.default.yellow.bold('tsd reinstall --save') + '. If this fails run the commands ' + 'yourself. TSD must be installed via `npm install -g tsd`.');
        this.spawnCommand('tsd', ['reinstall', '--save']);
      }
    }
  },
  end: function end() {
    this.composeWith('ng-poly:module', {
      args: ['home'],
      options: {
        module: 'home',
        markup: this.markup,
        style: this.style,
        'test-dir': this.testDir,
        'test-script': this.testScript,
        'controller-as': this.controllerAs,
        'skip-controller': this.skipController,
        'ng-route': this.ngRoute
      }
    }, {
      local: require.resolve('../module'),
      link: 'strong'
    });
  }
});