'use strict';

require('babel-polyfill');

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _genBase = require('../genBase');

var _genBase2 = _interopRequireDefault(_genBase);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _ngAddDep = require('ng-add-dep');

var _ngAddDep2 = _interopRequireDefault(_ngAddDep);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _pify = require('pify');

var _pify2 = _interopRequireDefault(_pify);

var _utils = require('../utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

module.exports = _genBase2.default.extend({
  initialize: function initialize() {
    this.module = this.name;

    // used by genBase to know this is not a normal component for module-type structures
    // used for creating TypeScript reference path
    this.isModule = true;

    // if moduleName ends with a slash remove it
    if (this.module.charAt(this.module.length - 1) === '/' || this.module.charAt(this.module.length - 1) === '\\') {
      this.module = this.module.slice(0, this.module.length - 1);
    }
  },
  writing: function writing() {
    var done, filePath, filePathToCheck, depName, parentDir, parentModuleName, moduleDest;
    return regeneratorRuntime.async(function writing$(_context) {
      while (1) switch (_context.prev = _context.next) {
        case 0:
          done = this.async(), filePath = undefined, filePathToCheck = undefined, depName = undefined, parentDir = undefined, parentModuleName = undefined, moduleDest = undefined;
          _context.next = 3;
          return regeneratorRuntime.awrap(this.getConfig());

        case 3:
          this.context = _context.sent;

          // extract the new module name (last part of path)
          this.context.moduleName = _path2.default.basename(this.module);

          // modify context because these were all performed on the ENTIRE module path
          // instead of just the new module's name
          this.context.lowerCamel = _utils2.default.lowerCamel(this.context.moduleName);
          this.context.hyphenModule = _utils2.default.hyphenName(this.context.moduleName);
          this.context.upperModule = _utils2.default.upperCamel(this.context.moduleName);
          this.context.templateUrl = _path2.default.join(this.module).replace(/\\/g, '/');
          _context.next = 11;
          return regeneratorRuntime.awrap(_utils2.default.normalizeModulePath(this.module));

        case 11:
          this.context.modulePath = _context.sent;
          _context.next = 14;
          return regeneratorRuntime.awrap((0, _pify2.default)(_mkdirp2.default)(_path2.default.join(this.context.appDir, this.context.modulePath)));

        case 14:

          // check if path and moduleName are the same
          // if yes - get root app.js file to prepare adding dep
          // else - get parent module file to prepare adding dep
          if (this.context.moduleName === this.module) {
            filePathToCheck = _path2.default.join(this.context.appDir, 'app');
          } else {
            // go up one driectory from new module's path to retrieve parent's directory
            parentDir = _path2.default.join(this.context.appDir, this.context.modulePath, '..');
            // get parent module's name
            parentModuleName = _path2.default.basename(parentDir);

            filePathToCheck = _path2.default.join(this.context.appDir, this.context.modulePath, '..', parentModuleName);
          }

          // get module path
          _context.next = 17;
          return regeneratorRuntime.awrap(_utils2.default.findModuleFile(filePathToCheck));

        case 17:
          filePath = _context.sent;

          // if adding dep to app file, then dep is `module
          // else dep is `parent.module`
          depName = (this.context.parentModuleName ? this.context.parentModuleName + '.' : '') + this.context.lowerCamel;

          // add dep and save modifications
          _context.t0 = regeneratorRuntime;
          _context.t1 = filePath;
          _context.next = 23;
          return regeneratorRuntime.awrap((0, _pify2.default)(_fs2.default.readFile)(filePath, 'utf8'));

        case 23:
          _context.t2 = _context.sent;
          _context.t3 = depName;
          _context.t4 = (0, _ngAddDep2.default)(_context.t2, _context.t3);
          _context.t5 = (0, _pify2.default)(_fs2.default.writeFile)(_context.t1, _context.t4);
          _context.next = 29;
          return _context.t0.awrap.call(_context.t0, _context.t5);

        case 29:

          // create new module
          moduleDest = _path2.default.join(this.context.appDir, this.context.modulePath, this.context.hyphenModule);
          _context.next = 32;
          return regeneratorRuntime.awrap(this.copySrcFile('module', moduleDest + '-module.' + this.context.appScript, this.context));

        case 32:
          if (!(this.options && !this.options.empty)) {
            _context.next = 35;
            break;
          }

          _context.next = 35;
          return regeneratorRuntime.awrap(this.copySrcFile('module-routes', moduleDest + '-routes.' + this.context.appScript, this.context));

        case 35:

          done();

        case 36:
        case 'end':
          return _context.stop();
      }
    }, null, this);
  },
  end: function end() {
    if (this.options && !this.options.empty) {
      this.composeWith('ng-poly:route', {
        args: [_utils2.default.lowerCamel(this.module.split(/[\/\\]/).pop())],
        options: {
          module: this.module,
          url: '/' + this.context.hyphenModule,
          'template-url': this.context.modulePath + '/' + ('' + (this.context.structure === 'module-type' ? 'views/' : '') + this.context.hyphenModule + '.tpl.html'),

          markup: this.options.markup,
          'app-script': this.options['app-script'],
          'controller-as': this.options['controller-as'],
          'pass-func': this.options['pass-func'],
          'named-func': this.options['named-func'],
          'test-script': this.options['test-script'],
          'test-dir': this.options['test-dir'],
          'ng-route': this.options['ng-route'],
          style: this.options.style
        }
      }, {
        local: require.resolve('../route'),
        link: 'strong'
      });
    }
  }
});